/*!
\file sphere3d.cpp
\brief Class definition for Sphere3D
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date Feb 23, 2004
*/
#include <iostream>
#include <iomanip>
#include "sphere3d.h"
#include "timer.h"

using namespace std;


//=====================================================================================
//=====================================================================================
Sphere3D::Sphere3D() : Projection()
{
	faces.resize(12);
	for(int face = 0; face < 12; face++)
		faces[face].faceNumber(face);
	return;
}
//=====================================================================================
Sphere3D::~Sphere3D()
{
}

//=====================================================================================
void Sphere3D::setResolution(int sphere_res_)
{
	nside = sphere_res_;
	setThetas();
	for(int i = 0; i < 4; i++)
		faces[i].setRigging(nside,costhetas_np);
	for(int i = 4; i < 8; i++)
		faces[i].setRigging(nside,costhetas_eq);
	for(int i = 8; i < 12; i++)
		faces[i].setRigging(nside,costhetas_sp);
	sphere_res = sphere_res_;
	return;
}
//=====================================================================================
void Sphere3D::draw()
{
	for(int i = 0; i < 12; i++)
		faces[i].draw();
	return;
}
//=====================================================================================
void Sphere3D::setThetas()
{
	std::vector<double> thetas_np;	//!< theta values for North Pole Faces
	std::vector<double> thetas_eq;	//!< theta values for Equatorial Pole Faces
	std::vector<double> thetas_sp;	//!< theta values for South Pole Faces
	double theta,phi,costheta;
	int pix = 0;
	int dpix = 4;
	int npix = nside2npix(nside);
	const double eps = 0.128/nside;
	thetas_np.push_back(0.0);
	while( pix < npix ){
		pix2ang_ring(nside, pix,&theta,&phi);
		costheta = cos(theta);
		if( costheta > -eps )
			thetas_np.push_back(theta);
		if( fabs(costheta) <= (2./3.+eps) )
			thetas_eq.push_back(theta);
		if( costheta < eps )
			thetas_sp.push_back(theta);

		pix += dpix;
		if( costheta > 2./3.) {
			dpix += 4;
		}
		else if( costheta < -2./3. ) {
			dpix -= 4;
		}

	}
	thetas_sp.push_back(M_PI);
	//cout << "Nnp: " << thetas_np.size() << endl;
	//cout << "Enp: " << thetas_eq.size() << endl;
	//cout << "Snp: " << thetas_sp.size() << endl;

	unsigned int n = thetas_np.size();
	costhetas_np.resize(n);
	costhetas_eq.resize(n);
	costhetas_sp.resize(n);

	for(unsigned int i = 0; i < n; i++) {
		costhetas_np[i] = cos(thetas_np[n-i-1]);
		costhetas_eq[i] = cos(thetas_eq[n-i-1]);
		costhetas_sp[i] = cos(thetas_sp[n-i-1]);
	}
	return;
}
//=====================================================================================
bool Sphere3D::select(const qglviewer::Vec &o,const qglviewer::Vec &d, qglviewer::Vec &v)
{
	float sqrtterm = (o*d)*(o*d) + d*d - (d*d)*(o*o);
	if( sqrtterm < 0 )
		return false;
	float t1 = (-o*d - sqrt(sqrtterm))/(d*d);
	float t2 = (-o*d + sqrt(sqrtterm))/(d*d);
	float t = (t1 < t2) ? t1 : t2;
	v = o + t*d;
	return true;
}
